@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define _POSIX_SOURCE
@@ -135,7 +135,7 @@ cl_load_1_0 (int fd)
 			break;
 		
 		else if (r == CONTACT_RET) {	
-			*pc = malloc (sizeof (contact_t));
+			*pc = ct_new_contact ();
 			CHECK (*pc != NULL);
 				
 			r = read_tag (0, b, "hname", (*pc)->hname, MAX_HNAME);
@@ -148,18 +148,6 @@ cl_load_1_0 (int fd)
 						
 			if (r == 0) {
 				(*pc)->port = (port_t) strtoul (port, NULL, 10);
-				(*pc)->next = NULL;
-				(*pc)->state.ip = (ip_t)0;
-				(*pc)->state.socket = (sock_t)-1;
-				(*pc)->state.connected = FALSE;
-				(*pc)->state.hello_pending = TRUE;
-				(*pc)->state.in_buffer = NULL;
-				(*pc)->state.in_buf_size = 0;
-				(*pc)->state.in_buf_rcvd = 0;
-				(*pc)->state.out_buffer = NULL;
-				(*pc)->state.out_buf_size = 0;
-				(*pc)->state.hello_timeout = NULL;
-				(*pc)->state.ui_info = NULL;
 				pc = &((*pc)->next);
 			}
 			else {
@@ -255,7 +243,10 @@ cl_save ()
          
 	free (file);
 		
-	ftruncate (fd, 0);
+	if (ftruncate (fd, 0) < 0) {
+		close (fd);
+		return ERR;
+	}
 
 	r = write_tag (fd, "version", CONT_VERSION, 0);
 	if (r == ERR) {
@@ -378,7 +369,7 @@ cl_remove_contact (const char *nick)
 	while (*pc != NULL) {
 		if (strncmp (nick, (*pc)->nick, MAX_NICK) == 0) {
 			p = *pc;
-			if (p->state.hello_pending) {
+			if (!CT_IS_CONNECTED (p)) {
 				if (p->state.ui_info) {
 					/* the contact has a window */
 					ui_close_win (p);
