@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define _POSIX_SOURCE
@@ -93,16 +93,14 @@ pa_parse_line (contact_t *win_contact, const char *line)
 	
 	for (w = 0; w < MAX_W; w++)
 		words [w] = NULL;
-	
-	//ui_output_err (line);
-	
+
 	w = get_words (words, line, MAX_W);
 	co = command_id (words [0]+1, commands, COMMANDS_NUM);
 	
 	switch (co) {
 		case ADD_C:
 			if (w > 2) {
-				ct = malloc (sizeof (contact_t));
+				ct = ct_new_contact ();
 				if (ct == NULL) {
 					dmx_stop ();
 					return;
@@ -116,18 +114,6 @@ pa_parse_line (contact_t *win_contact, const char *line)
 					                             NULL, 10);
 				else
 					ct->port = DEFAULT_PORT;
-				ct->next = NULL;
-				ct->state.ip = 0;
-				ct->state.socket=-1;
-				ct->state.connected=FALSE;
-				ct->state.hello_pending=TRUE;
-				ct->state.in_buffer=NULL;
-				ct->state.in_buf_size=0;
-				ct->state.in_buf_rcvd=0;
-				ct->state.out_buffer=NULL;
-				ct->state.out_buf_size=0;
-				ct->state.hello_timeout=NULL;
-				ct->state.ui_info=NULL;
 				cl_add_contact (ct);
 			}
 			else
@@ -415,7 +401,7 @@ help (int c_id, char *opt)
 			ui_output_info ("For information on a command use "
 			                "/help <command>.");
 			ui_output_info ("Command name abbreviations are "
-			                "allowed (eg: /p).");
+			                "allowed (ex: /b).");
 			ui_output_info ("Text formatting: \\b\\\\b for bold, "
 			                "\\u\\\\u for underline\\u, \\\\1 to "
 			                "\\\\7 for colors, \\\\0 to reset.");
@@ -459,6 +445,7 @@ static void
 print_contact_info (contact_t *c)
 {
 	ip_t old_ip = c->state.ip;
+	char *str_state;
 
 	/* user may use /info to force IP resolution */
 	tr_resolv (c);
@@ -480,10 +467,21 @@ print_contact_info (contact_t *c)
 		                ipv4_to_string (c->state.ip));
 	}
 	ui_output_info ("\\b\\2Listen port:   \\0%u", c->port);
-	ui_output_info ("\\b\\2State:         \\0%s",
-	                (!c->state.connected) ? "Disconnected" :
-	                c->state.hello_pending ? "Waiting HELLO..."
-	                                       : "Connected");
+
+	str_state = "??? (this is a bug, please report it)";
+	switch (c->state.cn_state) {
+		case CS_OFFLINE: str_state = "Offline"; break;
+		case CS_HELLO_PENDING: str_state = "Waiting HELLO..."; break;
+		case CS_HANDSHAKING: str_state = "Handshaking"; break;
+		case CS_CONNECTED:
+			str_state = c->state.tls_info ? "Connected (SSL/TLS)"
+			                              : "Connected";
+			break;
+	}
+	ui_output_info ("\\b\\2State:         \\0%s", str_state);
+	if (CT_IS_CONNECTED (c)) {
+		tls_session_info (c);
+	}
 	ui_output_info ("");
 }
 
