@@ -13,12 +13,13 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define _POSIX_SOURCE
 
 #include <string.h>		/* strlen, strncmp, strcpy, memcpy */
+#include <stdlib.h>		/* malloc, free */
 #include <netinet/in.h>		/* htonl, htons, ntohl, ntohs */
 #include "protocol.h"
 #include "transport.h"
@@ -38,6 +39,8 @@ struct msg_header {
 		struct hdr_hello {
 			uint16_t version;    /* protocol version */
 			uint16_t port;       /* listen port */
+			uint8_t  tls_ena;    /* TLS enabled */
+			uint8_t  tls_req;    /* TLS required */
 		} hello;
 
 		/* Text message. The text is on the payload. */
@@ -70,7 +73,7 @@ enum { MSG_HELLO = 0, MSG_TEXT, MSG_BEEP /* ... */ };
 /* Prototypes */
 static int received_hello (contact_t *contact, struct hdr_hello *hdr,
                            const char *nick, int nick_len);
-static int received_text (contact_t *contact, const char *text, int len);
+static int received_text (contact_t *contact, const char *text);
 static int received_beep (contact_t *contact);
 static void hton_header ();
 static void ntoh_header ();
@@ -83,12 +86,13 @@ pr_send_text (contact_t *contact, const char *txt)
 
 	header.type = MSG_TEXT;
 	hton_header ();
-	tr_send_msg (contact, (BYTE *)&header, HEADER_SIZE(text),
-	             (BYTE *)txt, len);
 
 	if (cfg.logging) {
 		lg_log_text (cfg.nick, contact, txt, len);
-	}
+    	}
+
+	tr_send_msg (contact, (BYTE *)&header, HEADER_SIZE(text),
+	             (BYTE *)txt, len);
 }
 
 void
@@ -97,6 +101,8 @@ pr_send_hello (contact_t *contact)
 	header.type = MSG_HELLO;
 	header.c.hello.version = PROTOCOL_VERSION;
 	header.c.hello.port = cfg.listen_port;
+	header.c.hello.tls_ena = (uint8_t)tls_enabled ();
+	header.c.hello.tls_req = (uint8_t)0; /* TODO */
 	hton_header ();
 	tr_send_msg (contact, (BYTE *)&header, HEADER_SIZE(hello),
 	             (BYTE *)cfg.nick, strlen (cfg.nick));
@@ -105,7 +111,7 @@ pr_send_hello (contact_t *contact)
 void
 pr_send_beep (contact_t *contact)
 {
-	if (contact != BROADCAST && contact->state.hello_pending) {
+	if (contact != BROADCAST && !CT_IS_CONNECTED (contact)) {
 		ui_output_err ("Contact not connected.");
 		return;
 	}
@@ -146,8 +152,8 @@ pr_msg_received (contact_t *contact, const BYTE *msg, int len)
 			break;
 
 		case MSG_TEXT:
-			r = received_text (contact, (const char *)payload,
-			                   pl_size);
+			((char*)payload)[pl_size] = '\0';
+			r = received_text (contact, (const char *)payload);
 			break;
 
 		case MSG_BEEP:
@@ -179,6 +185,7 @@ received_hello (contact_t *contact, struct hdr_hello *hdr, const char *nick,
                 int nick_len)
 {
 	BOOL modif, modif_nick, hello_was_pending;
+	char decd_nick[MAX_NICK];
 
 	if (hdr->version < PROTOCOL_VERSION) {
 		ui_output_err ("Connection from outdated client! (from %s)",
@@ -193,16 +200,21 @@ received_hello (contact_t *contact, struct hdr_hello *hdr, const char *nick,
 		return ERR;
 	}
 
+	/*nick_len = utf8_decode ((char*) nick, nick_len, decd_nick, MAX_NICK);
+	  */
+	memcpy (decd_nick, nick, nick_len);
+
 	if (nick_len > (MAX_NICK - 1)) {
 		ui_output_err ("Contact's nick is too long (from %s).",
 		               ipv4_to_string (contact->state.ip));
 		return ERR;
 	}
+	decd_nick[nick_len] = '\0';
 
 	/* TODO We should check there are no control characters on the
 	   nick, or some jokes can be done. For example, let /nick only
 	   accept printable chars, and accept the same here. */
-	if (nick_len == 0 || nick[0] == '\0') {
+	if (nick_len == 0 || decd_nick[0] == '\0') {
 		ui_output_err ("Bad nick in incomming HELLO (from %s).",
 		               ipv4_to_string (contact->state.ip));
 		return ERR;
@@ -214,25 +226,21 @@ received_hello (contact_t *contact, struct hdr_hello *hdr, const char *nick,
 		modif = TRUE;
 	}
 
-	hello_was_pending = contact->state.hello_pending;
+	hello_was_pending = (contact->state.cn_state == CS_HELLO_PENDING);
 	if (hello_was_pending && contact->nick[0] != '\0') {
 		ui_output_info ("Contact %s connected.", contact->nick);
 	}
 
-	if (strlen (contact->nick) != nick_len ||
-	    strncmp (contact->nick, nick, nick_len) != 0) {
-		char new_nick[MAX_NICK];
-		memcpy (new_nick, nick, nick_len);
-		new_nick[nick_len] = '\0';
+	if (strcmp (contact->nick, decd_nick) != 0) {
 		if (contact->nick[0] != '\0') {
 			ui_output_info ("Contact %s is now known as %s.",
-			                contact->nick, new_nick);
+			                contact->nick, decd_nick);
 		}
 		else {
 			ui_output_info ("A new contact has connected: %s.",
-			                new_nick);
+			                decd_nick);
 		}
-		strcpy (contact->nick, new_nick);
+		strcpy (contact->nick, decd_nick);
 		modif = modif_nick = TRUE;
 	}
 
@@ -242,7 +250,13 @@ received_hello (contact_t *contact, struct hdr_hello *hdr, const char *nick,
 	if (hello_was_pending) {
 		dmx_remove_timer (contact->state.hello_timeout);
 		contact->state.hello_timeout = NULL;
-		contact->state.hello_pending = FALSE;
+		if (tls_enabled () && !hdr->tls_ena) {
+			ui_output_info ("\\bWarning:\\0 Contact doesn't "
+			                "have SSL/TLS enabled.");
+			tls_session_deinit (contact);
+			/* tls_handshake below will simply enter CS_CONNECTED */
+		}
+		tls_handshake (contact);
 	}
 
 	if (hello_was_pending || modif_nick) {
@@ -253,15 +267,15 @@ received_hello (contact_t *contact, struct hdr_hello *hdr, const char *nick,
 }
 
 static int
-received_text (contact_t *contact, const char *text, int len)
+received_text (contact_t *contact, const char *text)
 {
 	/* TODO Check there are only printable characters and no '\n', etc.
 	   Should we check for a maximum length? */
 
-	ui_output_msg (contact, text, len);
+	ui_output_msg (contact, text);
 
 	if (cfg.logging) {
-		lg_log_text (contact->nick, contact, text, len);
+		lg_log_text (contact->nick, contact, text, strlen(text));
 	}
 
 	return OK;
