@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define _POSIX_SOURCE
@@ -31,6 +31,7 @@
 #include "transport.h"
 #include "demux.h"
 #include "tcp.h"
+#include "tls.h"
 #include "contact_list.h"
 #include "protocol.h"
 #include "misc.h"
@@ -83,7 +84,7 @@ tr_connect (contact_t *contact)
 
 	CHECK (contact->state.ip && contact->state.socket < 0);
 
-	if (contact->state.connected) {
+	if (contact->state.cn_state != CS_OFFLINE) {
 		ui_output_err ("Program error in tr_connect. Please report. "
 		               "Connected without a socket!?");
 	}
@@ -114,8 +115,10 @@ tr_disconnect (contact_t *contact)
 	if (contact->state.socket >= 0) {
 		dmx_remove_input_fd (contact->state.socket);
 		dmx_remove_output_fd (contact->state.socket);
+		tls_session_finish (contact);
 		close (contact->state.socket);
 		contact->state.socket = -1;
+		tls_session_deinit (contact);
 	}
 
 	if (contact->state.in_buffer) {
@@ -131,8 +134,7 @@ tr_disconnect (contact_t *contact)
 	}
 	contact->state.in_buf_size = 0;
 
-	contact->state.connected = FALSE;
-	contact->state.hello_pending = TRUE;
+	contact->state.cn_state = CS_OFFLINE;
 
 	if (contact->nick[0] != '\0') {
 		ui_redraw_contacts ();
@@ -152,19 +154,27 @@ tr_send_msg (contact_t *contact, const BYTE *header, int hdr_len,
 	bcast = (contact == BROADCAST);
 	if (bcast) {
 		/* we'll build the full message in the first connected
-		   contact, and then we'll copy it to the rest; we look
-		   at state.socket and not state.hello_pending because
-		   e.g. if nick is changed when some connection has been
-		   established but hello didn't arrive, the contact
-		   would never know about the new nick */
+		   contact, and then we'll copy it to the rest */
+		/* TODO we have a problem here; if this is a nick-change-hello,
+		   connections in CS_HELLO_PENDING or CS_HANDSHAKING won't
+		   know about the nick-change! we cannot send it because
+		   we could interfer with TLS handshake...
+		   A nice solution would be to implement two HELLO's, one
+		   to use before handshake, including version & TLS information
+		   (even the public key signature to find the contact_t!),
+		   and another to use after handshake, including the nick.
+		   So, even the nick and all information we add to HELLO
+		   in the future, will be encrypted!
+		   (NOTE: this also affects broadcast 'for' below) */
 		for (contact = contacts;
-		     contact && !contact->state.connected;
+		     contact && !CT_IS_CONNECTED (contact);
 		     contact = contact->next)
 			;
 		if (!contact)
 			return;
 	}
-	else if (!contact->state.connected) {
+	else if (!CT_IS_CONNECTED (contact) &&
+	         contact->state.cn_state != CS_HELLO_PENDING) {
 		ui_output_err ("Trying to send data to not connected "
 		               "contact. This is a bug, please report it.");
 		return;
@@ -191,7 +201,7 @@ tr_send_msg (contact_t *contact, const BYTE *header, int hdr_len,
 		/* copy full msg to all other connected contacts */
 		for (contact = contact->next; contact;
 		     contact = contact->next) {
-			if (contact->state.connected) {
+			if (CT_IS_CONNECTED (contact)) {
 				was_void = (contact->state.out_buffer == NULL);
 				p = inc_out_buffer (contact, len_full_msg);
 				CHECK (p);
@@ -262,9 +272,11 @@ cb_server_data (int fd, void *data)
 		c = create_temp_contact (ip);
 		CHECK (c);
 	}
-	else if (c->state.connected) {
-		ui_output_info ("%s reconnected, closing old connection.",
-		                c->nick);
+	else if (c->state.cn_state != CS_OFFLINE) {
+		if (CT_IS_CONNECTED (c)) {
+			ui_output_info ("%s reconnected, closing old "
+			                "connection.", c->nick);
+		}
 		tr_disconnect (c);
 	}
 	else if (c->state.socket >= 0) {
@@ -274,8 +286,8 @@ cb_server_data (int fd, void *data)
 	}
 
 	c->state.socket = s;
-	c->state.connected = TRUE;
-	c->state.hello_pending = TRUE; /* TODO it should already be, check it */
+	c->state.cn_state = CS_HELLO_PENDING;
+	tls_session_init (c, TRUE);
 	dmx_add_input_fd (s, cb_net_input, (void *)c);
 	create_hello_timeout (c);
 	pr_send_hello (c);
@@ -290,6 +302,11 @@ cb_net_input (int fd, void *data)
 	contact_t *c = (contact_t *)data;
 	int ret;
 
+	if (c->state.cn_state == CS_HANDSHAKING) {
+		/* ignore data received during TLS handshake */
+		return;
+	}
+
 	ret = recv_msg (c);
 	if (ret == RECV_COMPLETE) {
 		ret = pr_msg_received (c, c->state.in_buffer,
@@ -303,6 +320,7 @@ cb_net_input (int fd, void *data)
 		c->state.in_buf_rcvd = 0;
 	}
 	else if (ret == RECV_ERROR) {
+		/* error or disconnection */
 		tr_disconnect (c);
 	}
 }
@@ -312,14 +330,15 @@ cb_out_connection (int fd, void *data)
 {
 	contact_t *c = (contact_t *)data;
 
-	if (fd != c->state.socket || c->state.connected) {
+	if (fd != c->state.socket || c->state.cn_state != CS_OFFLINE) {
 		ui_output_err ("Program error in cb_out_connection. Please "
 		               "report. fd=%d socket=%d.", fd, c->state.socket);
 	}
 	dmx_remove_output_fd (fd);
 
 	if (tcp_connect_result (fd) == OK) {
-		c->state.connected = TRUE;
+		tls_session_init (c, FALSE);
+		c->state.cn_state = CS_HELLO_PENDING;
 		dmx_add_input_fd (fd, cb_net_input, data);
 		create_hello_timeout (c);
 		pr_send_hello (c);
@@ -339,7 +358,7 @@ cb_net_output (int fd, void *data)
 	int b = 0;
 
 	if (buf) {
-		b = write (c->state.socket, buf, cur_size);
+		b = tls_send (c, buf, cur_size);
 		if (b < 0) {
 			ui_output_err ("Error sending data to %s.", c->nick);
 			b = 0;
@@ -378,7 +397,7 @@ recv_msg (contact_t *c)
 		   the next message */
 		rcvd = c->state.in_buf_rcvd;
 		// TODO CHECK (c->state.in_buf_rcvd <= sizeof (len_t));
-		b = net_read (s, ((BYTE *)(&(c->state.in_buf_size))) + rcvd,
+		b = tls_recv (c, ((BYTE *)(&(c->state.in_buf_size))) + rcvd,
 		              sizeof (len_t) - rcvd);
 		CHECK (b >= 0);
 
@@ -391,13 +410,13 @@ recv_msg (contact_t *c)
 		CHECK (c->state.in_buf_size > 0); /* TODO check limit? */
 
 		c->state.in_buf_rcvd = 0;
-		buf = c->state.in_buffer = malloc (c->state.in_buf_size);
+		buf = c->state.in_buffer = malloc (c->state.in_buf_size + 1);
 		CHECK_J (c->state.in_buffer, critical);
 	}
 
 	/* receive or continue receiving the message */
 	rcvd = c->state.in_buf_rcvd;
-	b = net_read (s, buf + rcvd, c->state.in_buf_size - rcvd);
+	b = tls_recv (c, buf + rcvd, c->state.in_buf_size - rcvd);
 	CHECK (b >= 0);
 	c->state.in_buf_rcvd += b;
 
@@ -456,24 +475,11 @@ create_temp_contact (ip_t ip)
 {
 	contact_t *c;
 
-	c = malloc (sizeof (contact_t));
+	c = ct_new_contact ();
 	CHECK_J (c, end);
 
 	strcpy (c->hname, ipv4_to_string (ip));
-	c->port = 0;
-	c->nick[0] = '\0';
-	c->next = NULL;
 	c->state.ip = ip;
-	c->state.socket = -1;
-	c->state.connected = FALSE;
-	c->state.hello_pending = TRUE;
-	c->state.in_buffer = NULL;
-	c->state.in_buf_size = 0;
-	c->state.in_buf_rcvd = 0;
-	c->state.out_buffer = NULL;
-	c->state.out_buf_size = 0;
-	c->state.hello_timeout = NULL;
-	c->state.ui_info = NULL;
 	cl_add_contact (c);
 
 end:
