@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define _POSIX_SOURCE
@@ -217,7 +217,10 @@ cf_save ()
          
 	free (file);
 		
-	ftruncate (fd, 0);
+	if (ftruncate (fd, 0) < 0) {
+		close (fd);
+		return ERR;
+	}
 
 	r = write_tag (fd, "version", CONF_VERSION, 0);
 	if (r == ERR) {
