@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define _POSIX_SOURCE
@@ -24,6 +24,7 @@
 #include <string.h>		/* memcpy, memcmp */
 #include "ui_common.h"
 #include "uconfig.h"
+#include "misc.h"
 
 /* Prototypes */
 static void input_left ();
@@ -41,8 +42,53 @@ static void update_text ();
 void
 cb_read_key (int fd, void *data)
 {
-	int     c = wgetch (win_input);
-	int     i;
+	wint_t  c;
+	int     i, r;
+
+	r = wget_wch (win_input, &c);
+	if (r == ERR) {
+		return;
+	}
+
+	if (r != KEY_CODE_YES) {
+		switch (c) {
+		case '\b':
+		case 0x7f:		/* ASCII DEL */
+			c = KEY_BACKSPACE;
+			r = KEY_CODE_YES;
+			break;
+		case '\n':
+			c = KEY_ENTER;
+			r = KEY_CODE_YES;
+			break;
+		case '\f':		/* CTRL+F */
+			remake_windows ();
+			return;
+		default:
+			if (cur->ibuf_len == IBUF_SIZE || iscntrl (c)) {
+				beep ();
+				return;
+			}
+
+			for (i = cur->ibuf_len - 1; i >= cur->icursor; i--)
+				cur->ibuf[i + 1] = cur->ibuf[i];
+
+			cur->ibuf_len++;
+			cur->ibuf[cur->icursor++] = c;
+
+			if (cur->icursor - cur->ishow >= i_input.ncols) {
+				cur->ishow++;
+			}
+			break;
+		}
+
+		if (r != KEY_CODE_YES) {
+			paint_input ();
+			wnoutrefresh (win_input);
+			doupdate ();
+			return;
+		}
+	}
 
 	if (c >= KEY_F(1) && c <= KEY_F(12)) {
 		change_window (c - KEY_F(1));
@@ -50,16 +96,8 @@ cb_read_key (int fd, void *data)
 	}
 
 	switch (c) {
-	case ERR:
 	case KEY_RESIZE:
 		return;
-
-		/*
-		   case CTL_D:
-		   case TAB:
-		   case KEY_F(1):
-		   case KEY_IC: ???
-		 */
 	case KEY_LEFT:
 		input_left ();
 		break;
@@ -74,15 +112,12 @@ cb_read_key (int fd, void *data)
 		break;
 	/* TODO erasechar()? */
 	case KEY_BACKSPACE:
-	case '\b':
-	case 0x7f:		/* ASCII DEL */
 		input_backspace ();
 		break;
 	case KEY_DC:
 		input_delete ();
 		break;
 	case KEY_ENTER:
-	case '\n':
 		input_enter ();
 		break;
 	case KEY_HOME:
@@ -101,25 +136,6 @@ cb_read_key (int fd, void *data)
 		move_text_index_down (&show_text_idx, i_text.nlines / 2);
 		update_text ();
 		break;
-	case '\f':		/* CTRL+F */
-		remake_windows ();
-		return;
-	default:
-		if (cur->ibuf_len == IBUF_SIZE || c > 255 || iscntrl (c)) {
-			beep ();
-			return;
-		}
-
-		for (i = cur->ibuf_len - 1; i >= cur->icursor; i--)
-			cur->ibuf[i + 1] = cur->ibuf[i];
-
-		cur->ibuf_len++;
-		cur->ibuf[cur->icursor++] = (char)c;
-
-		if (cur->icursor - cur->ishow >= i_input.ncols) {
-			cur->ishow++;
-		}
-		break;
 	}
 
 	paint_input ();
@@ -248,10 +264,10 @@ input_delete ()
 static void
 input_enter ()
 {
-	char *new_hist;
-	int   new_head;
-	BOOL  send_it = TRUE;
-	int   change_win = 0;
+	wchar_t *new_hist;
+	int      new_head;
+	BOOL     send_it = TRUE;
+	int      change_win = 0;
 
 	/* if the user is watching old text (PgUp), move to the bottom */
 	if (show_text_idx != cur->tbuf_head) {
@@ -260,7 +276,7 @@ input_enter ()
 
 	/* build and output the line (not for commands) */
 	if (cur->ibuf_len == 0 || cur->ibuf[0] != '/') {
-		if (!cur->contact || cur->contact->state.hello_pending) {
+		if (!(cur->contact && CT_IS_CONNECTED (cur->contact))) {
 			ui_output_err ("Contact not connected.");
 			send_it = FALSE;
 		}
@@ -273,9 +289,10 @@ input_enter ()
 	}
 
 	/* check if it's /<num> */
-	if (cur->ibuf_len > 1 && cur->ibuf[0] == '/' && isdigit(cur->ibuf[1])) {
-		cur->ibuf[cur->ibuf_len] = '\0';
-		change_win = atoi (&cur->ibuf[1]);
+	if (cur->ibuf_len > 1 && cur->ibuf[0] == '/' && iswdigit(cur->ibuf[1]))
+	{
+		cur->ibuf[cur->ibuf_len] = L'\0';
+		change_win = wcstol (&cur->ibuf[1], NULL, 10);
 		send_it = FALSE;
 	}
 
@@ -288,25 +305,28 @@ input_enter ()
 			cur->hist_tail = (new_head + 1) % MAX_HISTORY;
 		}
 		cur->hist[new_head] = cur->hist[cur->hist_head];
-		new_hist = malloc (cur->ibuf_len + 1);
+		new_hist = malloc ((cur->ibuf_len + 1) * sizeof(wchar_t));
 		if (!new_hist) {
 			dmx_stop ();
 			return;
 		}
-		memcpy (new_hist, cur->ibuf, cur->ibuf_len);
-		new_hist[cur->ibuf_len] = '\0';
+		memcpy (new_hist, cur->ibuf, cur->ibuf_len * sizeof(wchar_t));
+		new_hist[cur->ibuf_len] = L'\0';
 		cur->hist[cur->hist_head] = new_hist;
 		cur->hist_head = new_head;
 	}
 	cur->hist_pos = cur->hist_head;
 
 	if (send_it) {
-		/* TODO convert to UTF-8? (maybe in parse, when sending) */
+		char *utf8str;
 		/* the buffer is not null-terminated, but parse requires
 		   that; this is why ibuf is one byte longer */
-		cur->ibuf[cur->ibuf_len] = '\0';
+		cur->ibuf[cur->ibuf_len] = L'\0';
+		/* make utf-8 version */
+		utf8str = utf8_wencode (cur->ibuf);
 		/* send to parse */
-		pa_parse_line (cur->contact, cur->ibuf);
+		pa_parse_line (cur->contact, utf8str);
+		free (utf8str);
 	}
 
 	/* reset edit environment */
@@ -322,8 +342,8 @@ input_enter ()
 static void
 recover_hist ()
 {
-	int len = strlen (cur->hist[cur->hist_pos]);
-	memcpy (cur->ibuf, cur->hist[cur->hist_pos], len);
+	size_t len = wcslen (cur->hist[cur->hist_pos]);
+	memcpy (cur->ibuf, cur->hist[cur->hist_pos], len * sizeof(wchar_t));
 	cur->ibuf_len = len;
 	cur->icursor = len;
 	if (len < i_input.ncols)
@@ -337,7 +357,8 @@ recover_hist ()
 static inline BOOL
 repeated_input ()
 {
-	int prev, len;
+	int prev;
+	size_t len;
 
 	prev = cur->hist_head;
 	if (prev == cur->hist_tail)
@@ -347,9 +368,10 @@ repeated_input ()
 		prev = MAX_HISTORY;
 	prev--;
 
-	len = strlen (cur->hist[prev]);
+	len = wcslen (cur->hist[prev]);
 	return (len == cur->ibuf_len &&
-	        memcmp (cur->ibuf, cur->hist[prev], len) == 0);
+	        memcmp (cur->ibuf, cur->hist[prev], len * sizeof(wchar_t))
+	        == 0);
 }
 
 /* Repaint text, for PgUp/PgDn. */
