@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #define _POSIX_SOURCE
@@ -126,9 +126,6 @@ paint_input ()
 {
 	WINDOW *w = win_input;
 	int     y, x, i, show;
-#ifdef EDIT_NO_COLOR /* TODO document, parametrize or remove */
-	int     precurs;
-#endif
 
 	werase (w);
 	getmaxyx (w, y, x);
@@ -147,17 +144,11 @@ paint_input ()
 			show--;
 		}
 	}
-#ifdef EDIT_NO_COLOR /* TODO document or remove */
-	precurs = cur->icursor - cur->ishow;
-	mvwaddnstr (w, 1, precurs, &(cur->ibuf[cur->icursor]), show - precurs);
-	mvwaddnstr (w, 1, 0, &(cur->ibuf[cur->ishow]), precurs);
-#else
 	wmove (w, 1, 0);
 	proc_format (w, cur->ibuf, cur->ishow);
 	format_waddnstr (w, &(cur->ibuf[cur->ishow]), show, TRUE);
 	wstandend (w);
 	wmove (w, 1, cur->icursor - cur->ishow);
-#endif
 }
 
 /* Paint prompt (with local nick). */
@@ -218,10 +209,10 @@ paint_contacts ()
 				snprintf (stmp, x, "%s[%c%d] %s %s",
 				          i > 9 ? " " : "  ",
 				          i > 12 ? '/' : 'F', i, c->nick,
-				          (cw == vcur && c->state.hello_pending)
+				          (cw == vcur && !CT_IS_CONNECTED (c))
 					               ? "-<-" :
 				          (cw == vcur) ? "<-" :
-					  c->state.hello_pending ? "--" :
+					  (!CT_IS_CONNECTED (c)) ? "--" :
 					  cw->alarm ? "(*)" : "");
 				wcolor_set (w, COLOR_YELLOW, NULL);
 				waddnstr (w, stmp, 6);
