@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
-   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef USER_IFACE_H
@@ -41,7 +41,7 @@ extern void ui_close_win (contact_t *contact);
 extern void ui_close_cur_win ();
 
 /* Output a new message from a contact. */
-extern void ui_output_msg (contact_t *contact, const char *txt, int len);
+extern void ui_output_msg (contact_t *contact, const char *txt);
 
 /* Output an information message. Works like printf. */
 extern void ui_output_info (const char *fmt, ...);
